/**
 *
 * Magnolia Criteria API (http://www.openmindlab.com/lab/products/mgnlcriteria.html)
 * Copyright(C) 2009-2010, Openmind S.r.l. http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.sourceforge.openutils.mgnlcriteria.jcr.query;



/**
 * The result of an advanced jcr query.
 * @author fgiust
 * @version $Id: AdvancedResult.java 2854 2010-08-19 09:25:38Z fgiust $
 */
public interface AdvancedResult
{

    /**
     * Gets the maximum number of results per page
     * @return
     */
    int getItemsPerPage();

    /**
     * Gets the page number (1, 2, 3...)
     * @return
     */
    int getPage();

    /**
     * Gets the total number of results that would be retrieved without pagination
     * @return
     */
    int getTotalSize();

    /**
     * Gets the total number of pages
     * @return
     */
    int getNumberOfPages();

    /**
     * Gets the suggestion from the spell checker
     * @return
     */
    String getSpellCheckerSuggestion();

    /**
     * Gets an iterator over the results
     * @return
     */
    ResultIterator<AdvancedResultItem> getItems();

    public static AdvancedResult EMPTY_RESULT = new AdvancedResult()
    {

        public int getTotalSize()
        {
            return 0;
        }

        public String getSpellCheckerSuggestion()
        {
            return null;
        }

        public int getPage()
        {
            return 0;
        }

        public int getItemsPerPage()
        {
            return 0;
        }

        public ResultIterator<AdvancedResultItem> getItems()
        {
            // @todo return an empty result
            return null;
        }

        public int getNumberOfPages()
        {
            return 0;
        }
    };

}
